@@ -8,7 +8,7 @@
 #include <device/io_req.h>
 
 io_return_t kmsgopen (dev_t dev, int flag, io_req_t ior);
-io_return_t kmsgclose (dev_t dev, int flag);
+void kmsgclose (dev_t dev, int flag);
 io_return_t kmsgread (dev_t dev, io_req_t ior);
 io_return_t kmsggetstat (dev_t dev, int flavor,
 			 int *data, unsigned int *count);
