@@ -237,12 +237,12 @@ typedef mach_port_t	*processor_set_name_array_t;
 
 #ifdef KERNEL
 #if	MACH_HOST
-extern void	pset_sys_bootstrap(void);
 extern void	pset_sys_init(void);
 #endif	/* MACH_HOST */
 
 /* Pset internal functions */
 
+extern void	pset_sys_bootstrap(void);
 extern void	pset_reference(processor_set_t);
 extern void	pset_deallocate(processor_set_t);
 extern void	pset_remove_processor(processor_set_t, processor_t);
@@ -320,4 +320,10 @@ extern kern_return_t processor_set_threads(
 		natural_t	*count);
 #endif
 
+void processor_doaction(processor_t processor);
+void processor_doshutdown(processor_t processor);
+void quantum_set(processor_set_t pset);
+void pset_init(processor_set_t pset);
+void processor_init(processor_t pr, int slot_num);
+
 #endif	/* _KERN_PROCESSOR_H_ */
