@@ -61,6 +61,7 @@
 
 #include <sys/types.h>
 #include <machine/spl.h>
+#include <machine/vm_param.h>
 
 #include <mach/mach_types.h>
 #include <mach/kern_return.h>
@@ -69,6 +70,9 @@
 #include <mach/vm_param.h>
 #include <mach/notify.h>
 
+#include <kern/kalloc.h>
+#include <kern/printf.h>
+
 #include <ipc/ipc_port.h>
 #include <ipc/ipc_space.h>
 
@@ -83,8 +87,8 @@
 #include <device/if_hdr.h>
 #include <device/net_io.h>
 #include <device/device_reply.user.h>
-
-#include <linux_emul.h>
+#include <device/device_emul.h>
+#include <device/ds_routines.h>
 
 #define MACH_INCLUDE
 #include <linux/kernel.h>
@@ -98,7 +102,7 @@
 #include <linux/etherdevice.h>
 #include <linux/wireless.h>
 
-extern int linux_intr_pri;
+#include <linux/dev/glue/glue.h>
 
 /* One of these is associated with each instance of a device.  */
 struct net_data
@@ -249,7 +253,6 @@ void
 dev_kfree_skb (struct sk_buff *skb, int mode)
 {
   unsigned flags;
-  extern void *io_done_list;
 
   /* Queue sk_buff on done list if there is a
      page list attached or we need to send a reply.
@@ -399,8 +402,14 @@ device_open (ipc_port_t reply_port, mach_msg_type_name_t reply_port_type,
 	}
       else
 	{
-	  dev->flags |= LINUX_IFF_UP | LINUX_IFF_RUNNING;
+	  /* IPv6 heavily relies on multicasting (especially router and
+	     neighbor solicits and advertisements), so enable reception of
+	     those multicast packets by setting `LINUX_IFF_ALLMULTI'.  */
+	  dev->flags |= LINUX_IFF_UP | LINUX_IFF_RUNNING | LINUX_IFF_ALLMULTI;
 	  skb_queue_head_init (&dev->buffs[0]);
+
+	  if (dev->set_multicast_list)
+	    dev->set_multicast_list (dev);
 	}
       if (IP_VALID (reply_port))
 	ds_device_open_reply (reply_port, reply_port_type,
@@ -419,63 +428,43 @@ device_write (void *d, ipc_port_t reply_port,
 	      int *bytes_written)
 {
   unsigned char *p;
-  int i, amt, skblen, s;
-  io_return_t err = 0;
+  int i, s;
   vm_map_copy_t copy = (vm_map_copy_t) data;
+  char *map_data;
+  vm_offset_t map_addr;
+  vm_size_t map_size;
   struct net_data *nd = d;
   struct linux_device *dev = nd->dev;
   struct sk_buff *skb;
+  kern_return_t kr;
 
   if (count == 0 || count > dev->mtu + dev->hard_header_len)
     return D_INVALID_SIZE;
 
   /* Allocate a sk_buff.  */
-  amt = PAGE_SIZE - (copy->offset & PAGE_MASK);
-  skblen = (amt >= count) ? 0 : count;
-  skb = dev_alloc_skb (skblen);
+  skb = dev_alloc_skb (count);
   if (!skb)
     return D_NO_MEMORY;
 
-  /* Copy user data.  This is only required if it spans multiple pages.  */
-  if (skblen == 0)
-    {
-      assert (copy->cpy_npages == 1);
+  /* Map user data.  */
+  kr = kmem_io_map_copyout(device_io_map, (vm_offset_t *)&map_data,
+			   &map_addr, &map_size, copy, count);
 
-      skb->copy = copy;
-      skb->data = ((void *) copy->cpy_page_list[0]->phys_addr
-		   + (copy->offset & PAGE_MASK));
-      skb->len = count;
-      skb->head = skb->data;
-      skb->tail = skb->data + skb->len;
-      skb->end = skb->tail;
-    }
-  else
-    {
-      skb->len = skblen;
-      skb->tail = skb->data + skblen;
-      skb->end = skb->tail;
-      
-      memcpy (skb->data,
-	      ((void *) copy->cpy_page_list[0]->phys_addr
-	       + (copy->offset & PAGE_MASK)),
-	      amt);
-      count -= amt;
-      p = skb->data + amt;
-      for (i = 1; count > 0 && i < copy->cpy_npages; i++)
-	{
-	  amt = PAGE_SIZE;
-	  if (amt > count)
-	    amt = count;
-	  memcpy (p, (void *) copy->cpy_page_list[i]->phys_addr, amt);
-	  count -= amt;
-	  p += amt;
-	}
+  if (kr) {
+    dev_kfree_skb (skb, FREE_WRITE);
+    return D_NO_MEMORY;
+  }
 
-      assert (count == 0);
-
-      vm_map_copy_discard (copy);
-    }
+  /* XXX The underlying physical pages of the mapping could be highmem,
+     for which drivers require the use of a bounce buffer.  */
+  memcpy (skb->data, map_data, count);
+  kmem_io_map_deallocate (device_io_map, map_addr, map_size);
+  vm_map_copy_discard (copy);
 
+  skb->len = count;
+  skb->head = skb->data;
+  skb->tail = skb->data + skb->len;
+  skb->end = skb->tail;
   skb->dev = dev;
   skb->reply = reply_port;
   skb->reply_type = reply_port_type;
@@ -513,8 +502,10 @@ device_write (void *d, ipc_port_t reply_port,
                          + sizeof (struct packet_header);
         packet->type = header->ether_type;
         net_kmsg (kmsg)->sent = TRUE; /* Mark packet as sent.  */
+        s = splimp ();
         net_packet (&dev->net_data->ifnet, kmsg, packet->length,
                     ethernet_priority (kmsg));
+        splx (s);
       }
   }
 
@@ -526,6 +517,17 @@ static io_return_t
 device_get_status (void *d, dev_flavor_t flavor, dev_status_t status,
 		   mach_msg_type_number_t *count)
 {
+  if (flavor == NET_FLAGS)
+    {
+      struct net_data *net = (struct net_data *) d;
+
+      if (*count != 1)
+	return D_INVALID_SIZE;
+
+      status[0] = net->dev->flags;
+      return D_SUCCESS;
+    }
+
   if(flavor >= SIOCIWFIRST && flavor <= SIOCIWLAST)
     {
       /* handle wireless ioctl */
@@ -585,6 +587,21 @@ static io_return_t
 device_set_status(void *d, dev_flavor_t flavor, dev_status_t status,
 		  mach_msg_type_number_t count)
 {
+  if (flavor == NET_FLAGS)
+    {
+      if (count != 1)
+        return D_INVALID_SIZE;
+
+      short flags = status[0];
+      struct net_data *net = (struct net_data *) d;
+
+      dev_change_flags (net->dev, flags);
+
+      /* Change the flags of the Mach device, too. */
+      net->ifnet.if_flags = net->dev->flags;
+      return D_SUCCESS;
+    }
+
   if(flavor < SIOCIWFIRST || flavor > SIOCIWLAST)
     return D_INVALID_OPERATION;
 
