@@ -182,7 +182,7 @@ kern_return_t vm_statistics(
 	*stat = vm_stat;
 
 	stat->pagesize = PAGE_SIZE;
-	stat->free_count = vm_page_free_count;
+	stat->free_count = vm_page_mem_free();
 	stat->active_count = vm_page_active_count;
 	stat->inactive_count = vm_page_inactive_count;
 	stat->wire_count = vm_page_wire_count;
@@ -197,8 +197,8 @@ kern_return_t vm_cache_statistics(
 	if (map == VM_MAP_NULL)
 		return KERN_INVALID_ARGUMENT;
 
-	stats->cache_object_count = vm_object_cached_count;
-	stats->cache_count = vm_object_cached_pages;
+	stats->cache_object_count = vm_object_external_count;
+	stats->cache_count = vm_object_external_pages;
 
 	/* XXX Not implemented yet */
 	stats->active_tmp_count = 0;
